# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.bigtable_admin_v2.types import instance as gba_instance
from google.protobuf import field_mask_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.bigtable.admin.v2",
    manifest={
        "CreateInstanceRequest",
        "GetInstanceRequest",
        "ListInstancesRequest",
        "ListInstancesResponse",
        "PartialUpdateInstanceRequest",
        "DeleteInstanceRequest",
        "CreateClusterRequest",
        "GetClusterRequest",
        "ListClustersRequest",
        "ListClustersResponse",
        "DeleteClusterRequest",
        "CreateInstanceMetadata",
        "UpdateInstanceMetadata",
        "CreateClusterMetadata",
        "UpdateClusterMetadata",
        "PartialUpdateClusterMetadata",
        "PartialUpdateClusterRequest",
        "CreateAppProfileRequest",
        "GetAppProfileRequest",
        "ListAppProfilesRequest",
        "ListAppProfilesResponse",
        "UpdateAppProfileRequest",
        "DeleteAppProfileRequest",
        "UpdateAppProfileMetadata",
        "ListHotTabletsRequest",
        "ListHotTabletsResponse",
        "CreateLogicalViewRequest",
        "CreateLogicalViewMetadata",
        "GetLogicalViewRequest",
        "ListLogicalViewsRequest",
        "ListLogicalViewsResponse",
        "UpdateLogicalViewRequest",
        "UpdateLogicalViewMetadata",
        "DeleteLogicalViewRequest",
        "CreateMaterializedViewRequest",
        "CreateMaterializedViewMetadata",
        "GetMaterializedViewRequest",
        "ListMaterializedViewsRequest",
        "ListMaterializedViewsResponse",
        "UpdateMaterializedViewRequest",
        "UpdateMaterializedViewMetadata",
        "DeleteMaterializedViewRequest",
    },
)


class CreateInstanceRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.CreateInstance.

    Attributes:
        parent (str):
            Required. The unique name of the project in which to create
            the new instance. Values are of the form
            ``projects/{project}``.
        instance_id (str):
            Required. The ID to be used when referring to the new
            instance within its project, e.g., just ``myinstance``
            rather than ``projects/myproject/instances/myinstance``.
        instance (google.cloud.bigtable_admin_v2.types.Instance):
            Required. The instance to create. Fields marked
            ``OutputOnly`` must be left blank.
        clusters (MutableMapping[str, google.cloud.bigtable_admin_v2.types.Cluster]):
            Required. The clusters to be created within the instance,
            mapped by desired cluster ID, e.g., just ``mycluster``
            rather than
            ``projects/myproject/instances/myinstance/clusters/mycluster``.
            Fields marked ``OutputOnly`` must be left blank.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    instance_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    instance: gba_instance.Instance = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_instance.Instance,
    )
    clusters: MutableMapping[str, gba_instance.Cluster] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=4,
        message=gba_instance.Cluster,
    )


class GetInstanceRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.GetInstance.

    Attributes:
        name (str):
            Required. The unique name of the requested instance. Values
            are of the form ``projects/{project}/instances/{instance}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListInstancesRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.ListInstances.

    Attributes:
        parent (str):
            Required. The unique name of the project for which a list of
            instances is requested. Values are of the form
            ``projects/{project}``.
        page_token (str):
            DEPRECATED: This field is unused and ignored.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ListInstancesResponse(proto.Message):
    r"""Response message for BigtableInstanceAdmin.ListInstances.

    Attributes:
        instances (MutableSequence[google.cloud.bigtable_admin_v2.types.Instance]):
            The list of requested instances.
        failed_locations (MutableSequence[str]):
            Locations from which Instance information could not be
            retrieved, due to an outage or some other transient
            condition. Instances whose Clusters are all in one of the
            failed locations may be missing from ``instances``, and
            Instances with at least one Cluster in a failed location may
            only have partial information returned. Values are of the
            form ``projects/<project>/locations/<zone_id>``
        next_page_token (str):
            DEPRECATED: This field is unused and ignored.
    """

    @property
    def raw_page(self):
        return self

    instances: MutableSequence[gba_instance.Instance] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_instance.Instance,
    )
    failed_locations: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class PartialUpdateInstanceRequest(proto.Message):
    r"""Request message for
    BigtableInstanceAdmin.PartialUpdateInstance.

    Attributes:
        instance (google.cloud.bigtable_admin_v2.types.Instance):
            Required. The Instance which will (partially)
            replace the current value.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The subset of Instance fields which
            should be replaced. Must be explicitly set.
    """

    instance: gba_instance.Instance = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_instance.Instance,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class DeleteInstanceRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.DeleteInstance.

    Attributes:
        name (str):
            Required. The unique name of the instance to be deleted.
            Values are of the form
            ``projects/{project}/instances/{instance}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateClusterRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.CreateCluster.

    Attributes:
        parent (str):
            Required. The unique name of the instance in which to create
            the new cluster. Values are of the form
            ``projects/{project}/instances/{instance}``.
        cluster_id (str):
            Required. The ID to be used when referring to the new
            cluster within its instance, e.g., just ``mycluster`` rather
            than
            ``projects/myproject/instances/myinstance/clusters/mycluster``.
        cluster (google.cloud.bigtable_admin_v2.types.Cluster):
            Required. The cluster to be created. Fields marked
            ``OutputOnly`` must be left blank.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    cluster_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    cluster: gba_instance.Cluster = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_instance.Cluster,
    )


class GetClusterRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.GetCluster.

    Attributes:
        name (str):
            Required. The unique name of the requested cluster. Values
            are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListClustersRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.ListClusters.

    Attributes:
        parent (str):
            Required. The unique name of the instance for which a list
            of clusters is requested. Values are of the form
            ``projects/{project}/instances/{instance}``. Use
            ``{instance} = '-'`` to list Clusters for all Instances in a
            project, e.g., ``projects/myproject/instances/-``.
        page_token (str):
            DEPRECATED: This field is unused and ignored.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ListClustersResponse(proto.Message):
    r"""Response message for BigtableInstanceAdmin.ListClusters.

    Attributes:
        clusters (MutableSequence[google.cloud.bigtable_admin_v2.types.Cluster]):
            The list of requested clusters.
        failed_locations (MutableSequence[str]):
            Locations from which Cluster information could not be
            retrieved, due to an outage or some other transient
            condition. Clusters from these locations may be missing from
            ``clusters``, or may only have partial information returned.
            Values are of the form
            ``projects/<project>/locations/<zone_id>``
        next_page_token (str):
            DEPRECATED: This field is unused and ignored.
    """

    @property
    def raw_page(self):
        return self

    clusters: MutableSequence[gba_instance.Cluster] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_instance.Cluster,
    )
    failed_locations: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=2,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class DeleteClusterRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.DeleteCluster.

    Attributes:
        name (str):
            Required. The unique name of the cluster to be deleted.
            Values are of the form
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class CreateInstanceMetadata(proto.Message):
    r"""The metadata for the Operation returned by CreateInstance.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.CreateInstanceRequest):
            The request that prompted the initiation of
            this CreateInstance operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
    """

    original_request: "CreateInstanceRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="CreateInstanceRequest",
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class UpdateInstanceMetadata(proto.Message):
    r"""The metadata for the Operation returned by UpdateInstance.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.PartialUpdateInstanceRequest):
            The request that prompted the initiation of
            this UpdateInstance operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
    """

    original_request: "PartialUpdateInstanceRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="PartialUpdateInstanceRequest",
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class CreateClusterMetadata(proto.Message):
    r"""The metadata for the Operation returned by CreateCluster.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.CreateClusterRequest):
            The request that prompted the initiation of
            this CreateCluster operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
        tables (MutableMapping[str, google.cloud.bigtable_admin_v2.types.CreateClusterMetadata.TableProgress]):
            Keys: the full ``name`` of each table that existed in the
            instance when CreateCluster was first called, i.e.
            ``projects/<project>/instances/<instance>/tables/<table>``.
            Any table added to the instance by a later API call will be
            created in the new cluster by that API call, not this one.

            Values: information on how much of a table's data has been
            copied to the newly-created cluster so far.
    """

    class TableProgress(proto.Message):
        r"""Progress info for copying a table's data to the new cluster.

        Attributes:
            estimated_size_bytes (int):
                Estimate of the size of the table to be
                copied.
            estimated_copied_bytes (int):
                Estimate of the number of bytes copied so far for this
                table. This will eventually reach 'estimated_size_bytes'
                unless the table copy is CANCELLED.
            state (google.cloud.bigtable_admin_v2.types.CreateClusterMetadata.TableProgress.State):

        """

        class State(proto.Enum):
            r"""

            Values:
                STATE_UNSPECIFIED (0):
                    No description available.
                PENDING (1):
                    The table has not yet begun copying to the
                    new cluster.
                COPYING (2):
                    The table is actively being copied to the new
                    cluster.
                COMPLETED (3):
                    The table has been fully copied to the new
                    cluster.
                CANCELLED (4):
                    The table was deleted before it finished
                    copying to the new cluster. Note that tables
                    deleted after completion will stay marked as
                    COMPLETED, not CANCELLED.
            """
            STATE_UNSPECIFIED = 0
            PENDING = 1
            COPYING = 2
            COMPLETED = 3
            CANCELLED = 4

        estimated_size_bytes: int = proto.Field(
            proto.INT64,
            number=2,
        )
        estimated_copied_bytes: int = proto.Field(
            proto.INT64,
            number=3,
        )
        state: "CreateClusterMetadata.TableProgress.State" = proto.Field(
            proto.ENUM,
            number=4,
            enum="CreateClusterMetadata.TableProgress.State",
        )

    original_request: "CreateClusterRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="CreateClusterRequest",
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    tables: MutableMapping[str, TableProgress] = proto.MapField(
        proto.STRING,
        proto.MESSAGE,
        number=4,
        message=TableProgress,
    )


class UpdateClusterMetadata(proto.Message):
    r"""The metadata for the Operation returned by UpdateCluster.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.Cluster):
            The request that prompted the initiation of
            this UpdateCluster operation.
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
    """

    original_request: gba_instance.Cluster = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_instance.Cluster,
    )
    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class PartialUpdateClusterMetadata(proto.Message):
    r"""The metadata for the Operation returned by
    PartialUpdateCluster.

    Attributes:
        request_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the original request was
            received.
        finish_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which the operation failed or was
            completed successfully.
        original_request (google.cloud.bigtable_admin_v2.types.PartialUpdateClusterRequest):
            The original request for
            PartialUpdateCluster.
    """

    request_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=1,
        message=timestamp_pb2.Timestamp,
    )
    finish_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    original_request: "PartialUpdateClusterRequest" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="PartialUpdateClusterRequest",
    )


class PartialUpdateClusterRequest(proto.Message):
    r"""Request message for
    BigtableInstanceAdmin.PartialUpdateCluster.

    Attributes:
        cluster (google.cloud.bigtable_admin_v2.types.Cluster):
            Required. The Cluster which contains the partial updates to
            be applied, subject to the update_mask.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The subset of Cluster fields which
            should be replaced.
    """

    cluster: gba_instance.Cluster = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_instance.Cluster,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class CreateAppProfileRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.CreateAppProfile.

    Attributes:
        parent (str):
            Required. The unique name of the instance in which to create
            the new app profile. Values are of the form
            ``projects/{project}/instances/{instance}``.
        app_profile_id (str):
            Required. The ID to be used when referring to the new app
            profile within its instance, e.g., just ``myprofile`` rather
            than
            ``projects/myproject/instances/myinstance/appProfiles/myprofile``.
        app_profile (google.cloud.bigtable_admin_v2.types.AppProfile):
            Required. The app profile to be created. Fields marked
            ``OutputOnly`` will be ignored.
        ignore_warnings (bool):
            If true, ignore safety checks when creating
            the app profile.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    app_profile_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    app_profile: gba_instance.AppProfile = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_instance.AppProfile,
    )
    ignore_warnings: bool = proto.Field(
        proto.BOOL,
        number=4,
    )


class GetAppProfileRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.GetAppProfile.

    Attributes:
        name (str):
            Required. The unique name of the requested app profile.
            Values are of the form
            ``projects/{project}/instances/{instance}/appProfiles/{app_profile}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListAppProfilesRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.ListAppProfiles.

    Attributes:
        parent (str):
            Required. The unique name of the instance for which a list
            of app profiles is requested. Values are of the form
            ``projects/{project}/instances/{instance}``. Use
            ``{instance} = '-'`` to list AppProfiles for all Instances
            in a project, e.g., ``projects/myproject/instances/-``.
        page_size (int):
            Maximum number of results per page.

            A page_size of zero lets the server choose the number of
            items to return. A page_size which is strictly positive will
            return at most that many items. A negative page_size will
            cause an error.

            Following the first request, subsequent paginated calls are
            not required to pass a page_size. If a page_size is set in
            subsequent calls, it must match the page_size given in the
            first request.
        page_token (str):
            The value of ``next_page_token`` returned by a previous
            call.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=3,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class ListAppProfilesResponse(proto.Message):
    r"""Response message for BigtableInstanceAdmin.ListAppProfiles.

    Attributes:
        app_profiles (MutableSequence[google.cloud.bigtable_admin_v2.types.AppProfile]):
            The list of requested app profiles.
        next_page_token (str):
            Set if not all app profiles could be returned in a single
            response. Pass this value to ``page_token`` in another
            request to get the next page of results.
        failed_locations (MutableSequence[str]):
            Locations from which AppProfile information could not be
            retrieved, due to an outage or some other transient
            condition. AppProfiles from these locations may be missing
            from ``app_profiles``. Values are of the form
            ``projects/<project>/locations/<zone_id>``
    """

    @property
    def raw_page(self):
        return self

    app_profiles: MutableSequence[gba_instance.AppProfile] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_instance.AppProfile,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )
    failed_locations: MutableSequence[str] = proto.RepeatedField(
        proto.STRING,
        number=3,
    )


class UpdateAppProfileRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.UpdateAppProfile.

    Attributes:
        app_profile (google.cloud.bigtable_admin_v2.types.AppProfile):
            Required. The app profile which will
            (partially) replace the current value.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Required. The subset of app profile fields
            which should be replaced. If unset, all fields
            will be replaced.
        ignore_warnings (bool):
            If true, ignore safety checks when updating
            the app profile.
    """

    app_profile: gba_instance.AppProfile = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_instance.AppProfile,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )
    ignore_warnings: bool = proto.Field(
        proto.BOOL,
        number=3,
    )


class DeleteAppProfileRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.DeleteAppProfile.

    Attributes:
        name (str):
            Required. The unique name of the app profile to be deleted.
            Values are of the form
            ``projects/{project}/instances/{instance}/appProfiles/{app_profile}``.
        ignore_warnings (bool):
            Required. If true, ignore safety checks when
            deleting the app profile.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    ignore_warnings: bool = proto.Field(
        proto.BOOL,
        number=2,
    )


class UpdateAppProfileMetadata(proto.Message):
    r"""The metadata for the Operation returned by UpdateAppProfile."""


class ListHotTabletsRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.ListHotTablets.

    Attributes:
        parent (str):
            Required. The cluster name to list hot tablets. Value is in
            the following form:
            ``projects/{project}/instances/{instance}/clusters/{cluster}``.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The start time to list hot tablets. The hot
            tablets in the response will have start times
            between the requested start time and end time.
            Start time defaults to Now if it is unset, and
            end time defaults to Now - 24 hours if it is
            unset. The start time should be less than the
            end time, and the maximum allowed time range
            between start time and end time is 48 hours.
            Start time and end time should have values
            between Now and Now - 14 days.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            The end time to list hot tablets.
        page_size (int):
            Maximum number of results per page.

            A page_size that is empty or zero lets the server choose the
            number of items to return. A page_size which is strictly
            positive will return at most that many items. A negative
            page_size will cause an error.

            Following the first request, subsequent paginated calls do
            not need a page_size field. If a page_size is set in
            subsequent calls, it must match the page_size given in the
            first request.
        page_token (str):
            The value of ``next_page_token`` returned by a previous
            call.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=4,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ListHotTabletsResponse(proto.Message):
    r"""Response message for BigtableInstanceAdmin.ListHotTablets.

    Attributes:
        hot_tablets (MutableSequence[google.cloud.bigtable_admin_v2.types.HotTablet]):
            List of hot tablets in the tables of the
            requested cluster that fall within the requested
            time range. Hot tablets are ordered by node cpu
            usage percent. If there are multiple hot tablets
            that correspond to the same tablet within a
            15-minute interval, only the hot tablet with the
            highest node cpu usage will be included in the
            response.
        next_page_token (str):
            Set if not all hot tablets could be returned in a single
            response. Pass this value to ``page_token`` in another
            request to get the next page of results.
    """

    @property
    def raw_page(self):
        return self

    hot_tablets: MutableSequence[gba_instance.HotTablet] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_instance.HotTablet,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CreateLogicalViewRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.CreateLogicalView.

    Attributes:
        parent (str):
            Required. The parent instance where this logical view will
            be created. Format:
            ``projects/{project}/instances/{instance}``.
        logical_view_id (str):
            Required. The ID to use for the logical view,
            which will become the final component of the
            logical view's resource name.
        logical_view (google.cloud.bigtable_admin_v2.types.LogicalView):
            Required. The logical view to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    logical_view_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    logical_view: gba_instance.LogicalView = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_instance.LogicalView,
    )


class CreateLogicalViewMetadata(proto.Message):
    r"""The metadata for the Operation returned by CreateLogicalView.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.CreateLogicalViewRequest):
            The request that prompted the initiation of
            this CreateLogicalView operation.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was canceled.
    """

    original_request: "CreateLogicalViewRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="CreateLogicalViewRequest",
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class GetLogicalViewRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.GetLogicalView.

    Attributes:
        name (str):
            Required. The unique name of the requested logical view.
            Values are of the form
            ``projects/{project}/instances/{instance}/logicalViews/{logical_view}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListLogicalViewsRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.ListLogicalViews.

    Attributes:
        parent (str):
            Required. The unique name of the instance for which the list
            of logical views is requested. Values are of the form
            ``projects/{project}/instances/{instance}``.
        page_size (int):
            Optional. The maximum number of logical views
            to return. The service may return fewer than
            this value
        page_token (str):
            Optional. A page token, received from a previous
            ``ListLogicalViews`` call. Provide this to retrieve the
            subsequent page.

            When paginating, all other parameters provided to
            ``ListLogicalViews`` must match the call that provided the
            page token.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListLogicalViewsResponse(proto.Message):
    r"""Response message for BigtableInstanceAdmin.ListLogicalViews.

    Attributes:
        logical_views (MutableSequence[google.cloud.bigtable_admin_v2.types.LogicalView]):
            The list of requested logical views.
        next_page_token (str):
            A token, which can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    logical_views: MutableSequence[gba_instance.LogicalView] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_instance.LogicalView,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class UpdateLogicalViewRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.UpdateLogicalView.

    Attributes:
        logical_view (google.cloud.bigtable_admin_v2.types.LogicalView):
            Required. The logical view to update.

            The logical view's ``name`` field is used to identify the
            view to update. Format:
            ``projects/{project}/instances/{instance}/logicalViews/{logical_view}``.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. The list of fields to update.
    """

    logical_view: gba_instance.LogicalView = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_instance.LogicalView,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class UpdateLogicalViewMetadata(proto.Message):
    r"""The metadata for the Operation returned by UpdateLogicalView.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.UpdateLogicalViewRequest):
            The request that prompted the initiation of
            this UpdateLogicalView operation.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation was started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was canceled.
    """

    original_request: "UpdateLogicalViewRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="UpdateLogicalViewRequest",
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class DeleteLogicalViewRequest(proto.Message):
    r"""Request message for BigtableInstanceAdmin.DeleteLogicalView.

    Attributes:
        name (str):
            Required. The unique name of the logical view to be deleted.
            Format:
            ``projects/{project}/instances/{instance}/logicalViews/{logical_view}``.
        etag (str):
            Optional. The current etag of the logical
            view. If an etag is provided and does not match
            the current etag of the logical view, deletion
            will be blocked and an ABORTED error will be
            returned.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=2,
    )


class CreateMaterializedViewRequest(proto.Message):
    r"""Request message for
    BigtableInstanceAdmin.CreateMaterializedView.

    Attributes:
        parent (str):
            Required. The parent instance where this materialized view
            will be created. Format:
            ``projects/{project}/instances/{instance}``.
        materialized_view_id (str):
            Required. The ID to use for the materialized
            view, which will become the final component of
            the materialized view's resource name.
        materialized_view (google.cloud.bigtable_admin_v2.types.MaterializedView):
            Required. The materialized view to create.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    materialized_view_id: str = proto.Field(
        proto.STRING,
        number=2,
    )
    materialized_view: gba_instance.MaterializedView = proto.Field(
        proto.MESSAGE,
        number=3,
        message=gba_instance.MaterializedView,
    )


class CreateMaterializedViewMetadata(proto.Message):
    r"""The metadata for the Operation returned by
    CreateMaterializedView.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.CreateMaterializedViewRequest):
            The request that prompted the initiation of
            this CreateMaterializedView operation.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was canceled.
    """

    original_request: "CreateMaterializedViewRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="CreateMaterializedViewRequest",
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class GetMaterializedViewRequest(proto.Message):
    r"""Request message for
    BigtableInstanceAdmin.GetMaterializedView.

    Attributes:
        name (str):
            Required. The unique name of the requested materialized
            view. Values are of the form
            ``projects/{project}/instances/{instance}/materializedViews/{materialized_view}``.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )


class ListMaterializedViewsRequest(proto.Message):
    r"""Request message for
    BigtableInstanceAdmin.ListMaterializedViews.

    Attributes:
        parent (str):
            Required. The unique name of the instance for which the list
            of materialized views is requested. Values are of the form
            ``projects/{project}/instances/{instance}``.
        page_size (int):
            Optional. The maximum number of materialized
            views to return. The service may return fewer
            than this value
        page_token (str):
            Optional. A page token, received from a previous
            ``ListMaterializedViews`` call. Provide this to retrieve the
            subsequent page.

            When paginating, all other parameters provided to
            ``ListMaterializedViews`` must match the call that provided
            the page token.
    """

    parent: str = proto.Field(
        proto.STRING,
        number=1,
    )
    page_size: int = proto.Field(
        proto.INT32,
        number=2,
    )
    page_token: str = proto.Field(
        proto.STRING,
        number=3,
    )


class ListMaterializedViewsResponse(proto.Message):
    r"""Response message for
    BigtableInstanceAdmin.ListMaterializedViews.

    Attributes:
        materialized_views (MutableSequence[google.cloud.bigtable_admin_v2.types.MaterializedView]):
            The list of requested materialized views.
        next_page_token (str):
            A token, which can be sent as ``page_token`` to retrieve the
            next page. If this field is omitted, there are no subsequent
            pages.
    """

    @property
    def raw_page(self):
        return self

    materialized_views: MutableSequence[
        gba_instance.MaterializedView
    ] = proto.RepeatedField(
        proto.MESSAGE,
        number=1,
        message=gba_instance.MaterializedView,
    )
    next_page_token: str = proto.Field(
        proto.STRING,
        number=2,
    )


class UpdateMaterializedViewRequest(proto.Message):
    r"""Request message for
    BigtableInstanceAdmin.UpdateMaterializedView.

    Attributes:
        materialized_view (google.cloud.bigtable_admin_v2.types.MaterializedView):
            Required. The materialized view to update.

            The materialized view's ``name`` field is used to identify
            the view to update. Format:
            ``projects/{project}/instances/{instance}/materializedViews/{materialized_view}``.
        update_mask (google.protobuf.field_mask_pb2.FieldMask):
            Optional. The list of fields to update.
    """

    materialized_view: gba_instance.MaterializedView = proto.Field(
        proto.MESSAGE,
        number=1,
        message=gba_instance.MaterializedView,
    )
    update_mask: field_mask_pb2.FieldMask = proto.Field(
        proto.MESSAGE,
        number=2,
        message=field_mask_pb2.FieldMask,
    )


class UpdateMaterializedViewMetadata(proto.Message):
    r"""The metadata for the Operation returned by
    UpdateMaterializedView.

    Attributes:
        original_request (google.cloud.bigtable_admin_v2.types.UpdateMaterializedViewRequest):
            The request that prompted the initiation of
            this UpdateMaterializedView operation.
        start_time (google.protobuf.timestamp_pb2.Timestamp):
            The time at which this operation was started.
        end_time (google.protobuf.timestamp_pb2.Timestamp):
            If set, the time at which this operation
            finished or was canceled.
    """

    original_request: "UpdateMaterializedViewRequest" = proto.Field(
        proto.MESSAGE,
        number=1,
        message="UpdateMaterializedViewRequest",
    )
    start_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=2,
        message=timestamp_pb2.Timestamp,
    )
    end_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=3,
        message=timestamp_pb2.Timestamp,
    )


class DeleteMaterializedViewRequest(proto.Message):
    r"""Request message for
    BigtableInstanceAdmin.DeleteMaterializedView.

    Attributes:
        name (str):
            Required. The unique name of the materialized view to be
            deleted. Format:
            ``projects/{project}/instances/{instance}/materializedViews/{materialized_view}``.
        etag (str):
            Optional. The current etag of the
            materialized view. If an etag is provided and
            does not match the current etag of the
            materialized view, deletion will be blocked and
            an ABORTED error will be returned.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=2,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
import logging as std_logging
from collections import OrderedDict
import re
from typing import (
    Dict,
    Callable,
    Mapping,
    MutableMapping,
    MutableSequence,
    Optional,
    AsyncIterable,
    Awaitable,
    Sequence,
    Tuple,
    Type,
    Union,
)

from google.cloud.bigtable_v2 import gapic_version as package_version

from google.api_core.client_options import ClientOptions
from google.api_core import exceptions as core_exceptions
from google.api_core import gapic_v1
from google.api_core import retry_async as retries
from google.auth import credentials as ga_credentials  # type: ignore
from google.oauth2 import service_account  # type: ignore
import google.protobuf


try:
    OptionalRetry = Union[retries.AsyncRetry, gapic_v1.method._MethodDefault, None]
except AttributeError:  # pragma: NO COVER
    OptionalRetry = Union[retries.AsyncRetry, object, None]  # type: ignore

from google.cloud.bigtable_v2.types import bigtable
from google.cloud.bigtable_v2.types import data
from google.cloud.bigtable_v2.types import request_stats
from google.protobuf import timestamp_pb2  # type: ignore
from .transports.base import BigtableTransport, DEFAULT_CLIENT_INFO
from .transports.grpc_asyncio import BigtableGrpcAsyncIOTransport
from .client import BigtableClient

try:
    from google.api_core import client_logging  # type: ignore

    CLIENT_LOGGING_SUPPORTED = True  # pragma: NO COVER
except ImportError:  # pragma: NO COVER
    CLIENT_LOGGING_SUPPORTED = False

_LOGGER = std_logging.getLogger(__name__)


class BigtableAsyncClient:
    """Service for reading from and writing to existing Bigtable
    tables.
    """

    _client: BigtableClient

    # Copy defaults from the synchronous client for use here.
    # Note: DEFAULT_ENDPOINT is deprecated. Use _DEFAULT_ENDPOINT_TEMPLATE instead.
    DEFAULT_ENDPOINT = BigtableClient.DEFAULT_ENDPOINT
    DEFAULT_MTLS_ENDPOINT = BigtableClient.DEFAULT_MTLS_ENDPOINT
    _DEFAULT_ENDPOINT_TEMPLATE = BigtableClient._DEFAULT_ENDPOINT_TEMPLATE
    _DEFAULT_UNIVERSE = BigtableClient._DEFAULT_UNIVERSE

    authorized_view_path = staticmethod(BigtableClient.authorized_view_path)
    parse_authorized_view_path = staticmethod(BigtableClient.parse_authorized_view_path)
    instance_path = staticmethod(BigtableClient.instance_path)
    parse_instance_path = staticmethod(BigtableClient.parse_instance_path)
    materialized_view_path = staticmethod(BigtableClient.materialized_view_path)
    parse_materialized_view_path = staticmethod(
        BigtableClient.parse_materialized_view_path
    )
    table_path = staticmethod(BigtableClient.table_path)
    parse_table_path = staticmethod(BigtableClient.parse_table_path)
    common_billing_account_path = staticmethod(
        BigtableClient.common_billing_account_path
    )
    parse_common_billing_account_path = staticmethod(
        BigtableClient.parse_common_billing_account_path
    )
    common_folder_path = staticmethod(BigtableClient.common_folder_path)
    parse_common_folder_path = staticmethod(BigtableClient.parse_common_folder_path)
    common_organization_path = staticmethod(BigtableClient.common_organization_path)
    parse_common_organization_path = staticmethod(
        BigtableClient.parse_common_organization_path
    )
    common_project_path = staticmethod(BigtableClient.common_project_path)
    parse_common_project_path = staticmethod(BigtableClient.parse_common_project_path)
    common_location_path = staticmethod(BigtableClient.common_location_path)
    parse_common_location_path = staticmethod(BigtableClient.parse_common_location_path)

    @classmethod
    def from_service_account_info(cls, info: dict, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            info.

        Args:
            info (dict): The service account private key info.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            BigtableAsyncClient: The constructed client.
        """
        return BigtableClient.from_service_account_info.__func__(BigtableAsyncClient, info, *args, **kwargs)  # type: ignore

    @classmethod
    def from_service_account_file(cls, filename: str, *args, **kwargs):
        """Creates an instance of this client using the provided credentials
            file.

        Args:
            filename (str): The path to the service account private key json
                file.
            args: Additional arguments to pass to the constructor.
            kwargs: Additional arguments to pass to the constructor.

        Returns:
            BigtableAsyncClient: The constructed client.
        """
        return BigtableClient.from_service_account_file.__func__(BigtableAsyncClient, filename, *args, **kwargs)  # type: ignore

    from_service_account_json = from_service_account_file

    @classmethod
    def get_mtls_endpoint_and_cert_source(
        cls, client_options: Optional[ClientOptions] = None
    ):
        """Return the API endpoint and client cert source for mutual TLS.

        The client cert source is determined in the following order:
        (1) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is not "true", the
        client cert source is None.
        (2) if `client_options.client_cert_source` is provided, use the provided one; if the
        default client cert source exists, use the default one; otherwise the client cert
        source is None.

        The API endpoint is determined in the following order:
        (1) if `client_options.api_endpoint` if provided, use the provided one.
        (2) if `GOOGLE_API_USE_CLIENT_CERTIFICATE` environment variable is "always", use the
        default mTLS endpoint; if the environment variable is "never", use the default API
        endpoint; otherwise if client cert source exists, use the default mTLS endpoint, otherwise
        use the default API endpoint.

        More details can be found at https://google.aip.dev/auth/4114.

        Args:
            client_options (google.api_core.client_options.ClientOptions): Custom options for the
                client. Only the `api_endpoint` and `client_cert_source` properties may be used
                in this method.

        Returns:
            Tuple[str, Callable[[], Tuple[bytes, bytes]]]: returns the API endpoint and the
                client cert source to use.

        Raises:
            google.auth.exceptions.MutualTLSChannelError: If any errors happen.
        """
        return BigtableClient.get_mtls_endpoint_and_cert_source(client_options)  # type: ignore

    @property
    def transport(self) -> BigtableTransport:
        """Returns the transport used by the client instance.

        Returns:
            BigtableTransport: The transport used by the client instance.
        """
        return self._client.transport

    @property
    def api_endpoint(self):
        """Return the API endpoint used by the client instance.

        Returns:
            str: The API endpoint used by the client instance.
        """
        return self._client._api_endpoint

    @property
    def universe_domain(self) -> str:
        """Return the universe domain used by the client instance.

        Returns:
            str: The universe domain used
                by the client instance.
        """
        return self._client._universe_domain

    get_transport_class = BigtableClient.get_transport_class

    def __init__(
        self,
        *,
        credentials: Optional[ga_credentials.Credentials] = None,
        transport: Optional[
            Union[str, BigtableTransport, Callable[..., BigtableTransport]]
        ] = "grpc_asyncio",
        client_options: Optional[ClientOptions] = None,
        client_info: gapic_v1.client_info.ClientInfo = DEFAULT_CLIENT_INFO,
    ) -> None:
        """Instantiates the bigtable async client.

        Args:
            credentials (Optional[google.auth.credentials.Credentials]): The
                authorization credentials to attach to requests. These
                credentials identify the application to the service; if none
                are specified, the client will attempt to ascertain the
                credentials from the environment.
            transport (Optional[Union[str,BigtableTransport,Callable[..., BigtableTransport]]]):
                The transport to use, or a Callable that constructs and returns a new transport to use.
                If a Callable is given, it will be called with the same set of initialization
                arguments as used in the BigtableTransport constructor.
                If set to None, a transport is chosen automatically.
            client_options (Optional[Union[google.api_core.client_options.ClientOptions, dict]]):
                Custom options for the client.

                1. The ``api_endpoint`` property can be used to override the
                default endpoint provided by the client when ``transport`` is
                not explicitly provided. Only if this property is not set and
                ``transport`` was not explicitly provided, the endpoint is
                determined by the GOOGLE_API_USE_MTLS_ENDPOINT environment
                variable, which have one of the following values:
                "always" (always use the default mTLS endpoint), "never" (always
                use the default regular endpoint) and "auto" (auto-switch to the
                default mTLS endpoint if client certificate is present; this is
                the default value).

                2. If the GOOGLE_API_USE_CLIENT_CERTIFICATE environment variable
                is "true", then the ``client_cert_source`` property can be used
                to provide a client certificate for mTLS transport. If
                not provided, the default SSL client certificate will be used if
                present. If GOOGLE_API_USE_CLIENT_CERTIFICATE is "false" or not
                set, no client certificate will be used.

                3. The ``universe_domain`` property can be used to override the
                default "googleapis.com" universe. Note that ``api_endpoint``
                property still takes precedence; and ``universe_domain`` is
                currently not supported for mTLS.

            client_info (google.api_core.gapic_v1.client_info.ClientInfo):
                The client info used to send a user-agent string along with
                API requests. If ``None``, then default info will be used.
                Generally, you only need to set this if you're developing
                your own client library.

        Raises:
            google.auth.exceptions.MutualTlsChannelError: If mutual TLS transport
                creation failed for any reason.
        """
        self._client = BigtableClient(
            credentials=credentials,
            transport=transport,
            client_options=client_options,
            client_info=client_info,
        )

        if CLIENT_LOGGING_SUPPORTED and _LOGGER.isEnabledFor(
            std_logging.DEBUG
        ):  # pragma: NO COVER
            _LOGGER.debug(
                "Created client `google.bigtable_v2.BigtableAsyncClient`.",
                extra={
                    "serviceName": "google.bigtable.v2.Bigtable",
                    "universeDomain": getattr(
                        self._client._transport._credentials, "universe_domain", ""
                    ),
                    "credentialsType": f"{type(self._client._transport._credentials).__module__}.{type(self._client._transport._credentials).__qualname__}",
                    "credentialsInfo": getattr(
                        self.transport._credentials, "get_cred_info", lambda: None
                    )(),
                }
                if hasattr(self._client._transport, "_credentials")
                else {
                    "serviceName": "google.bigtable.v2.Bigtable",
                    "credentialsType": None,
                },
            )

    def read_rows(
        self,
        request: Optional[Union[bigtable.ReadRowsRequest, dict]] = None,
        *,
        table_name: Optional[str] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> Awaitable[AsyncIterable[bigtable.ReadRowsResponse]]:
        r"""Streams back the contents of all requested rows in
        key order, optionally applying the same Reader filter to
        each. Depending on their size, rows and cells may be
        broken up across multiple responses, but atomicity of
        each row will still be preserved. See the
        ReadRowsResponse documentation for details.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.ReadRowsRequest, dict]]):
                The request object. Request message for
                Bigtable.ReadRows.
            table_name (:class:`str`):
                Optional. The unique name of the table from which to
                read.

                Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            AsyncIterable[google.cloud.bigtable_v2.types.ReadRowsResponse]:
                Response message for
                Bigtable.ReadRows.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [table_name, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.ReadRowsRequest):
            request = bigtable.ReadRowsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.read_rows
        ]

        header_params = {}

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)$"
        )
        regex_match = routing_param_regex.match(request.table_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.authorized_view_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        routing_param_regex = re.compile(
            "^(?P<name>projects/[^/]+/instances/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.materialized_view_name)
        if regex_match and regex_match.group("name"):
            header_params["name"] = regex_match.group("name")

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def sample_row_keys(
        self,
        request: Optional[Union[bigtable.SampleRowKeysRequest, dict]] = None,
        *,
        table_name: Optional[str] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> Awaitable[AsyncIterable[bigtable.SampleRowKeysResponse]]:
        r"""Returns a sample of row keys in the table. The
        returned row keys will delimit contiguous sections of
        the table of approximately equal size, which can be used
        to break up the data for distributed tasks like
        mapreduces.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.SampleRowKeysRequest, dict]]):
                The request object. Request message for
                Bigtable.SampleRowKeys.
            table_name (:class:`str`):
                Optional. The unique name of the table from which to
                sample row keys.

                Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            AsyncIterable[google.cloud.bigtable_v2.types.SampleRowKeysResponse]:
                Response message for
                Bigtable.SampleRowKeys.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [table_name, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.SampleRowKeysRequest):
            request = bigtable.SampleRowKeysRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.sample_row_keys
        ]

        header_params = {}

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)$"
        )
        regex_match = routing_param_regex.match(request.table_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.authorized_view_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        routing_param_regex = re.compile(
            "^(?P<name>projects/[^/]+/instances/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.materialized_view_name)
        if regex_match and regex_match.group("name"):
            header_params["name"] = regex_match.group("name")

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def mutate_row(
        self,
        request: Optional[Union[bigtable.MutateRowRequest, dict]] = None,
        *,
        table_name: Optional[str] = None,
        row_key: Optional[bytes] = None,
        mutations: Optional[MutableSequence[data.Mutation]] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> bigtable.MutateRowResponse:
        r"""Mutates a row atomically. Cells already present in the row are
        left unchanged unless explicitly changed by ``mutation``.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.MutateRowRequest, dict]]):
                The request object. Request message for
                Bigtable.MutateRow.
            table_name (:class:`str`):
                Optional. The unique name of the table to which the
                mutation should be applied.

                Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            row_key (:class:`bytes`):
                Required. The key of the row to which
                the mutation should be applied.

                This corresponds to the ``row_key`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            mutations (:class:`MutableSequence[google.cloud.bigtable_v2.types.Mutation]`):
                Required. Changes to be atomically
                applied to the specified row. Entries
                are applied in order, meaning that
                earlier mutations can be masked by later
                ones. Must contain at least one entry
                and at most 100000.

                This corresponds to the ``mutations`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.bigtable_v2.types.MutateRowResponse:
                Response message for
                Bigtable.MutateRow.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [table_name, row_key, mutations, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.MutateRowRequest):
            request = bigtable.MutateRowRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if row_key is not None:
            request.row_key = row_key
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id
        if mutations:
            request.mutations.extend(mutations)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.mutate_row
        ]

        header_params = {}

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)$"
        )
        regex_match = routing_param_regex.match(request.table_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.authorized_view_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def mutate_rows(
        self,
        request: Optional[Union[bigtable.MutateRowsRequest, dict]] = None,
        *,
        table_name: Optional[str] = None,
        entries: Optional[MutableSequence[bigtable.MutateRowsRequest.Entry]] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> Awaitable[AsyncIterable[bigtable.MutateRowsResponse]]:
        r"""Mutates multiple rows in a batch. Each individual row
        is mutated atomically as in MutateRow, but the entire
        batch is not executed atomically.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.MutateRowsRequest, dict]]):
                The request object. Request message for
                BigtableService.MutateRows.
            table_name (:class:`str`):
                Optional. The unique name of the table to which the
                mutations should be applied.

                Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            entries (:class:`MutableSequence[google.cloud.bigtable_v2.types.MutateRowsRequest.Entry]`):
                Required. The row keys and
                corresponding mutations to be applied in
                bulk. Each entry is applied as an atomic
                mutation, but the entries may be applied
                in arbitrary order (even between entries
                for the same row). At least one entry
                must be specified, and in total the
                entries can contain at most 100000
                mutations.

                This corresponds to the ``entries`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            AsyncIterable[google.cloud.bigtable_v2.types.MutateRowsResponse]:
                Response message for
                BigtableService.MutateRows.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [table_name, entries, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.MutateRowsRequest):
            request = bigtable.MutateRowsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id
        if entries:
            request.entries.extend(entries)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.mutate_rows
        ]

        header_params = {}

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)$"
        )
        regex_match = routing_param_regex.match(request.table_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.authorized_view_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def check_and_mutate_row(
        self,
        request: Optional[Union[bigtable.CheckAndMutateRowRequest, dict]] = None,
        *,
        table_name: Optional[str] = None,
        row_key: Optional[bytes] = None,
        predicate_filter: Optional[data.RowFilter] = None,
        true_mutations: Optional[MutableSequence[data.Mutation]] = None,
        false_mutations: Optional[MutableSequence[data.Mutation]] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> bigtable.CheckAndMutateRowResponse:
        r"""Mutates a row atomically based on the output of a
        predicate Reader filter.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.CheckAndMutateRowRequest, dict]]):
                The request object. Request message for
                Bigtable.CheckAndMutateRow.
            table_name (:class:`str`):
                Optional. The unique name of the table to which the
                conditional mutation should be applied.

                Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            row_key (:class:`bytes`):
                Required. The key of the row to which
                the conditional mutation should be
                applied.

                This corresponds to the ``row_key`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            predicate_filter (:class:`google.cloud.bigtable_v2.types.RowFilter`):
                The filter to be applied to the contents of the
                specified row. Depending on whether or not any results
                are yielded, either ``true_mutations`` or
                ``false_mutations`` will be executed. If unset, checks
                that the row contains any values at all.

                This corresponds to the ``predicate_filter`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            true_mutations (:class:`MutableSequence[google.cloud.bigtable_v2.types.Mutation]`):
                Changes to be atomically applied to the specified row if
                ``predicate_filter`` yields at least one cell when
                applied to ``row_key``. Entries are applied in order,
                meaning that earlier mutations can be masked by later
                ones. Must contain at least one entry if
                ``false_mutations`` is empty, and at most 100000.

                This corresponds to the ``true_mutations`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            false_mutations (:class:`MutableSequence[google.cloud.bigtable_v2.types.Mutation]`):
                Changes to be atomically applied to the specified row if
                ``predicate_filter`` does not yield any cells when
                applied to ``row_key``. Entries are applied in order,
                meaning that earlier mutations can be masked by later
                ones. Must contain at least one entry if
                ``true_mutations`` is empty, and at most 100000.

                This corresponds to the ``false_mutations`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.bigtable_v2.types.CheckAndMutateRowResponse:
                Response message for
                Bigtable.CheckAndMutateRow.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [
            table_name,
            row_key,
            predicate_filter,
            true_mutations,
            false_mutations,
            app_profile_id,
        ]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.CheckAndMutateRowRequest):
            request = bigtable.CheckAndMutateRowRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if row_key is not None:
            request.row_key = row_key
        if predicate_filter is not None:
            request.predicate_filter = predicate_filter
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id
        if true_mutations:
            request.true_mutations.extend(true_mutations)
        if false_mutations:
            request.false_mutations.extend(false_mutations)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.check_and_mutate_row
        ]

        header_params = {}

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)$"
        )
        regex_match = routing_param_regex.match(request.table_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.authorized_view_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def ping_and_warm(
        self,
        request: Optional[Union[bigtable.PingAndWarmRequest, dict]] = None,
        *,
        name: Optional[str] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> bigtable.PingAndWarmResponse:
        r"""Warm up associated instance metadata for this
        connection. This call is not required but may be useful
        for connection keep-alive.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.PingAndWarmRequest, dict]]):
                The request object. Request message for client connection
                keep-alive and warming.
            name (:class:`str`):
                Required. The unique name of the instance to check
                permissions for as well as respond. Values are of the
                form ``projects/<project>/instances/<instance>``.

                This corresponds to the ``name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.bigtable_v2.types.PingAndWarmResponse:
                Response message for
                Bigtable.PingAndWarm connection
                keepalive and warming.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [name, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.PingAndWarmRequest):
            request = bigtable.PingAndWarmRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if name is not None:
            request.name = name
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.ping_and_warm
        ]

        header_params = {}

        routing_param_regex = re.compile("^(?P<name>projects/[^/]+/instances/[^/]+)$")
        regex_match = routing_param_regex.match(request.name)
        if regex_match and regex_match.group("name"):
            header_params["name"] = regex_match.group("name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def read_modify_write_row(
        self,
        request: Optional[Union[bigtable.ReadModifyWriteRowRequest, dict]] = None,
        *,
        table_name: Optional[str] = None,
        row_key: Optional[bytes] = None,
        rules: Optional[MutableSequence[data.ReadModifyWriteRule]] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> bigtable.ReadModifyWriteRowResponse:
        r"""Modifies a row atomically on the server. The method
        reads the latest existing timestamp and value from the
        specified columns and writes a new entry based on
        pre-defined read/modify/write rules. The new value for
        the timestamp is the greater of the existing timestamp
        or the current server time. The method returns the new
        contents of all modified cells.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.ReadModifyWriteRowRequest, dict]]):
                The request object. Request message for
                Bigtable.ReadModifyWriteRow.
            table_name (:class:`str`):
                Optional. The unique name of the table to which the
                read/modify/write rules should be applied.

                Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            row_key (:class:`bytes`):
                Required. The key of the row to which
                the read/modify/write rules should be
                applied.

                This corresponds to the ``row_key`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            rules (:class:`MutableSequence[google.cloud.bigtable_v2.types.ReadModifyWriteRule]`):
                Required. Rules specifying how the
                specified row's contents are to be
                transformed into writes. Entries are
                applied in order, meaning that earlier
                rules will affect the results of later
                ones. At least one entry must be
                specified, and there can be at most
                100000 rules.

                This corresponds to the ``rules`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.bigtable_v2.types.ReadModifyWriteRowResponse:
                Response message for
                Bigtable.ReadModifyWriteRow.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [table_name, row_key, rules, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.ReadModifyWriteRowRequest):
            request = bigtable.ReadModifyWriteRowRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if row_key is not None:
            request.row_key = row_key
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id
        if rules:
            request.rules.extend(rules)

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.read_modify_write_row
        ]

        header_params = {}

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)$"
        )
        regex_match = routing_param_regex.match(request.table_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        routing_param_regex = re.compile(
            "^(?P<table_name>projects/[^/]+/instances/[^/]+/tables/[^/]+)(?:/.*)?$"
        )
        regex_match = routing_param_regex.match(request.authorized_view_name)
        if regex_match and regex_match.group("table_name"):
            header_params["table_name"] = regex_match.group("table_name")

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def generate_initial_change_stream_partitions(
        self,
        request: Optional[
            Union[bigtable.GenerateInitialChangeStreamPartitionsRequest, dict]
        ] = None,
        *,
        table_name: Optional[str] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> Awaitable[
        AsyncIterable[bigtable.GenerateInitialChangeStreamPartitionsResponse]
    ]:
        r"""Returns the current list of partitions that make up the table's
        change stream. The union of partitions will cover the entire
        keyspace. Partitions can be read with ``ReadChangeStream``.
        NOTE: This API is only intended to be used by Apache Beam
        BigtableIO.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.GenerateInitialChangeStreamPartitionsRequest, dict]]):
                The request object. NOTE: This API is intended to be used
                by Apache Beam BigtableIO. Request
                message for
                Bigtable.GenerateInitialChangeStreamPartitions.
            table_name (:class:`str`):
                Required. The unique name of the table from which to get
                change stream partitions. Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.
                Change streaming must be enabled on the table.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used. Single cluster routing must be
                configured on the profile.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            AsyncIterable[google.cloud.bigtable_v2.types.GenerateInitialChangeStreamPartitionsResponse]:
                NOTE: This API is intended to be used
                by Apache Beam BigtableIO. Response
                message for
                Bigtable.GenerateInitialChangeStreamPartitions.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [table_name, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(
            request, bigtable.GenerateInitialChangeStreamPartitionsRequest
        ):
            request = bigtable.GenerateInitialChangeStreamPartitionsRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.generate_initial_change_stream_partitions
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("table_name", request.table_name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def read_change_stream(
        self,
        request: Optional[Union[bigtable.ReadChangeStreamRequest, dict]] = None,
        *,
        table_name: Optional[str] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> Awaitable[AsyncIterable[bigtable.ReadChangeStreamResponse]]:
        r"""Reads changes from a table's change stream. Changes
        will reflect both user-initiated mutations and mutations
        that are caused by garbage collection.
        NOTE: This API is only intended to be used by Apache
        Beam BigtableIO.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.ReadChangeStreamRequest, dict]]):
                The request object. NOTE: This API is intended to be used
                by Apache Beam BigtableIO. Request
                message for Bigtable.ReadChangeStream.
            table_name (:class:`str`):
                Required. The unique name of the table from which to
                read a change stream. Values are of the form
                ``projects/<project>/instances/<instance>/tables/<table>``.
                Change streaming must be enabled on the table.

                This corresponds to the ``table_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                This value specifies routing for
                replication. If not specified, the
                "default" application profile will be
                used. Single cluster routing must be
                configured on the profile.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            AsyncIterable[google.cloud.bigtable_v2.types.ReadChangeStreamResponse]:
                NOTE: This API is intended to be used
                by Apache Beam BigtableIO. Response
                message for Bigtable.ReadChangeStream.

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [table_name, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.ReadChangeStreamRequest):
            request = bigtable.ReadChangeStreamRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if table_name is not None:
            request.table_name = table_name
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.read_change_stream
        ]

        # Certain fields should be provided within the metadata header;
        # add these here.
        metadata = tuple(metadata) + (
            gapic_v1.routing_header.to_grpc_metadata(
                (("table_name", request.table_name),)
            ),
        )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def prepare_query(
        self,
        request: Optional[Union[bigtable.PrepareQueryRequest, dict]] = None,
        *,
        instance_name: Optional[str] = None,
        query: Optional[str] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> bigtable.PrepareQueryResponse:
        r"""Prepares a GoogleSQL query for execution on a
        particular Bigtable instance.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.PrepareQueryRequest, dict]]):
                The request object. Request message for
                Bigtable.PrepareQuery
            instance_name (:class:`str`):
                Required. The unique name of the instance against which
                the query should be executed. Values are of the form
                ``projects/<project>/instances/<instance>``

                This corresponds to the ``instance_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            query (:class:`str`):
                Required. The query string.
                This corresponds to the ``query`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                Optional. This value specifies routing for preparing the
                query. Note that this ``app_profile_id`` is only used
                for preparing the query. The actual query execution will
                use the app profile specified in the
                ``ExecuteQueryRequest``. If not specified, the
                ``default`` application profile will be used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            google.cloud.bigtable_v2.types.PrepareQueryResponse:
                Response message for
                Bigtable.PrepareQueryResponse

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [instance_name, query, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.PrepareQueryRequest):
            request = bigtable.PrepareQueryRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if instance_name is not None:
            request.instance_name = instance_name
        if query is not None:
            request.query = query
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.prepare_query
        ]

        header_params = {}

        routing_param_regex = re.compile("^(?P<name>projects/[^/]+/instances/[^/]+)$")
        regex_match = routing_param_regex.match(request.instance_name)
        if regex_match and regex_match.group("name"):
            header_params["name"] = regex_match.group("name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = await rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    def execute_query(
        self,
        request: Optional[Union[bigtable.ExecuteQueryRequest, dict]] = None,
        *,
        instance_name: Optional[str] = None,
        query: Optional[str] = None,
        app_profile_id: Optional[str] = None,
        retry: OptionalRetry = gapic_v1.method.DEFAULT,
        timeout: Union[float, object] = gapic_v1.method.DEFAULT,
        metadata: Sequence[Tuple[str, Union[str, bytes]]] = (),
    ) -> Awaitable[AsyncIterable[bigtable.ExecuteQueryResponse]]:
        r"""Executes a SQL query against a particular Bigtable
        instance.

        Args:
            request (Optional[Union[google.cloud.bigtable_v2.types.ExecuteQueryRequest, dict]]):
                The request object. Request message for
                Bigtable.ExecuteQuery
            instance_name (:class:`str`):
                Required. The unique name of the instance against which
                the query should be executed. Values are of the form
                ``projects/<project>/instances/<instance>``

                This corresponds to the ``instance_name`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            query (:class:`str`):
                Required. The query string.

                Exactly one of ``query`` and ``prepared_query`` is
                required. Setting both or neither is an
                ``INVALID_ARGUMENT``.

                This corresponds to the ``query`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            app_profile_id (:class:`str`):
                Optional. This value specifies routing for replication.
                If not specified, the ``default`` application profile
                will be used.

                This corresponds to the ``app_profile_id`` field
                on the ``request`` instance; if ``request`` is provided, this
                should not be set.
            retry (google.api_core.retry_async.AsyncRetry): Designation of what errors, if any,
                should be retried.
            timeout (float): The timeout for this request.
            metadata (Sequence[Tuple[str, Union[str, bytes]]]): Key/value pairs which should be
                sent along with the request as metadata. Normally, each value must be of type `str`,
                but for metadata keys ending with the suffix `-bin`, the corresponding values must
                be of type `bytes`.

        Returns:
            AsyncIterable[google.cloud.bigtable_v2.types.ExecuteQueryResponse]:
                Response message for
                Bigtable.ExecuteQuery

        """
        # Create or coerce a protobuf request object.
        # - Quick check: If we got a request object, we should *not* have
        #   gotten any keyword arguments that map to the request.
        flattened_params = [instance_name, query, app_profile_id]
        has_flattened_params = (
            len([param for param in flattened_params if param is not None]) > 0
        )
        if request is not None and has_flattened_params:
            raise ValueError(
                "If the `request` argument is set, then none of "
                "the individual field arguments should be set."
            )

        # - Use the request object if provided (there's no risk of modifying the input as
        #   there are no flattened fields), or create one.
        if not isinstance(request, bigtable.ExecuteQueryRequest):
            request = bigtable.ExecuteQueryRequest(request)

        # If we have keyword arguments corresponding to fields on the
        # request, apply these.
        if instance_name is not None:
            request.instance_name = instance_name
        if query is not None:
            request.query = query
        if app_profile_id is not None:
            request.app_profile_id = app_profile_id

        # Wrap the RPC method; this adds retry and timeout information,
        # and friendly error handling.
        rpc = self._client._transport._wrapped_methods[
            self._client._transport.execute_query
        ]

        header_params = {}

        routing_param_regex = re.compile("^(?P<name>projects/[^/]+/instances/[^/]+)$")
        regex_match = routing_param_regex.match(request.instance_name)
        if regex_match and regex_match.group("name"):
            header_params["name"] = regex_match.group("name")

        if True:  # always attach app_profile_id, even if empty string
            header_params["app_profile_id"] = request.app_profile_id

        if header_params:
            metadata = tuple(metadata) + (
                gapic_v1.routing_header.to_grpc_metadata(header_params),
            )

        # Validate the universe domain.
        self._client._validate_universe_domain()

        # Send the request.
        response = rpc(
            request,
            retry=retry,
            timeout=timeout,
            metadata=metadata,
        )

        # Done; return the response.
        return response

    async def __aenter__(self) -> "BigtableAsyncClient":
        return self

    async def __aexit__(self, exc_type, exc, tb):
        await self.transport.close()


DEFAULT_CLIENT_INFO = gapic_v1.client_info.ClientInfo(
    gapic_version=package_version.__version__
)

if hasattr(DEFAULT_CLIENT_INFO, "protobuf_runtime_version"):  # pragma: NO COVER
    DEFAULT_CLIENT_INFO.protobuf_runtime_version = google.protobuf.__version__


__all__ = ("BigtableAsyncClient",)



# Copyright 2016, Google LLC All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

from __future__ import absolute_import

from google.cloud.spanner_v1 import __version__
from google.cloud.spanner_v1 import param_types
from google.cloud.spanner_v1 import Client
from google.cloud.spanner_v1 import KeyRange
from google.cloud.spanner_v1 import KeySet
from google.cloud.spanner_v1 import AbstractSessionPool
from google.cloud.spanner_v1 import BurstyPool
from google.cloud.spanner_v1 import FixedSizePool
from google.cloud.spanner_v1 import PingingPool
from google.cloud.spanner_v1 import TransactionPingingPool
from google.cloud.spanner_v1 import COMMIT_TIMESTAMP


__all__ = (
    # google.cloud.spanner
    "__version__",
    "param_types",
    # google.cloud.spanner_v1.client
    "Client",
    # google.cloud.spanner_v1.keyset
    "KeyRange",
    "KeySet",
    # google.cloud.spanner_v1.pool
    "AbstractSessionPool",
    "BurstyPool",
    "FixedSizePool",
    "PingingPool",
    "TransactionPingingPool",
    # local
    "COMMIT_TIMESTAMP",
)



# -*- coding: utf-8 -*-
# Copyright 2025 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
from __future__ import annotations

from typing import MutableMapping, MutableSequence

import proto  # type: ignore

from google.cloud.aiplatform_v1.types import encryption_spec as gca_encryption_spec
from google.cloud.aiplatform_v1.types import env_var
from google.cloud.aiplatform_v1.types import service_networking
from google.protobuf import struct_pb2  # type: ignore
from google.protobuf import timestamp_pb2  # type: ignore


__protobuf__ = proto.module(
    package="google.cloud.aiplatform.v1",
    manifest={
        "ReasoningEngineSpec",
        "ReasoningEngine",
    },
)


class ReasoningEngineSpec(proto.Message):
    r"""ReasoningEngine configurations

    .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

    Attributes:
        source_code_spec (google.cloud.aiplatform_v1.types.ReasoningEngineSpec.SourceCodeSpec):
            Deploy from source code files with a defined
            entrypoint.

            This field is a member of `oneof`_ ``deployment_source``.
        service_account (str):
            Optional. The service account that the
            Reasoning Engine artifact runs as. It should
            have "roles/storage.objectViewer" for reading
            the user project's Cloud Storage and
            "roles/aiplatform.user" for using Vertex
            extensions. If not specified, the Vertex AI
            Reasoning Engine Service Agent in the project
            will be used.

            This field is a member of `oneof`_ ``_service_account``.
        package_spec (google.cloud.aiplatform_v1.types.ReasoningEngineSpec.PackageSpec):
            Optional. User provided package spec of the ReasoningEngine.
            Ignored when users directly specify a deployment image
            through ``deployment_spec.first_party_image_override``, but
            keeping the field_behavior to avoid introducing breaking
            changes. The ``deployment_source`` field should not be set
            if ``package_spec`` is specified.
        deployment_spec (google.cloud.aiplatform_v1.types.ReasoningEngineSpec.DeploymentSpec):
            Optional. The specification of a Reasoning
            Engine deployment.
        class_methods (MutableSequence[google.protobuf.struct_pb2.Struct]):
            Optional. Declarations for object class
            methods in OpenAPI specification format.
        agent_framework (str):
            Optional. The OSS agent framework used to
            develop the agent. Currently supported values:
            "google-adk", "langchain", "langgraph", "ag2",
            "llama-index", "custom".
    """

    class PackageSpec(proto.Message):
        r"""User-provided package specification, containing pickled
        object and package requirements.

        Attributes:
            pickle_object_gcs_uri (str):
                Optional. The Cloud Storage URI of the
                pickled python object.
            dependency_files_gcs_uri (str):
                Optional. The Cloud Storage URI of the
                dependency files in tar.gz format.
            requirements_gcs_uri (str):
                Optional. The Cloud Storage URI of the ``requirements.txt``
                file
            python_version (str):
                Optional. The Python version. Supported
                values are 3.9, 3.10, 3.11, 3.12, 3.13. If not
                specified, the default value is 3.10.
        """

        pickle_object_gcs_uri: str = proto.Field(
            proto.STRING,
            number=1,
        )
        dependency_files_gcs_uri: str = proto.Field(
            proto.STRING,
            number=2,
        )
        requirements_gcs_uri: str = proto.Field(
            proto.STRING,
            number=3,
        )
        python_version: str = proto.Field(
            proto.STRING,
            number=4,
        )

    class DeploymentSpec(proto.Message):
        r"""The specification of a Reasoning Engine deployment.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            env (MutableSequence[google.cloud.aiplatform_v1.types.EnvVar]):
                Optional. Environment variables to be set
                with the Reasoning Engine deployment. The
                environment variables can be updated through the
                UpdateReasoningEngine API.
            secret_env (MutableSequence[google.cloud.aiplatform_v1.types.SecretEnvVar]):
                Optional. Environment variables where the
                value is a secret in Cloud Secret Manager.
                To use this feature, add 'Secret Manager Secret
                Accessor' role
                (roles/secretmanager.secretAccessor) to AI
                Platform Reasoning Engine Service Agent.
            psc_interface_config (google.cloud.aiplatform_v1.types.PscInterfaceConfig):
                Optional. Configuration for PSC-I.
            min_instances (int):
                Optional. The minimum number of application instances that
                will be kept running at all times. Defaults to 1. Range: [0,
                10].

                This field is a member of `oneof`_ ``_min_instances``.
            max_instances (int):
                Optional. The maximum number of application instances that
                can be launched to handle increased traffic. Defaults to
                100. Range: [1, 1000].

                If VPC-SC or PSC-I is enabled, the acceptable range is [1,
                100].

                This field is a member of `oneof`_ ``_max_instances``.
            resource_limits (MutableMapping[str, str]):
                Optional. Resource limits for each container. Only 'cpu' and
                'memory' keys are supported. Defaults to {"cpu": "4",
                "memory": "4Gi"}.

                - The only supported values for CPU are '1', '2', '4', '6'
                  and '8'. For more information, go to
                  https://cloud.google.com/run/docs/configuring/cpu.
                - The only supported values for memory are '1Gi', '2Gi', ...
                  '32 Gi'.
                - For required cpu on different memory values, go to
                  https://cloud.google.com/run/docs/configuring/memory-limits
            container_concurrency (int):
                Optional. Concurrency for each container and agent server.
                Recommended value: 2 \* cpu + 1. Defaults to 9.

                This field is a member of `oneof`_ ``_container_concurrency``.
        """

        env: MutableSequence[env_var.EnvVar] = proto.RepeatedField(
            proto.MESSAGE,
            number=1,
            message=env_var.EnvVar,
        )
        secret_env: MutableSequence[env_var.SecretEnvVar] = proto.RepeatedField(
            proto.MESSAGE,
            number=2,
            message=env_var.SecretEnvVar,
        )
        psc_interface_config: service_networking.PscInterfaceConfig = proto.Field(
            proto.MESSAGE,
            number=4,
            message=service_networking.PscInterfaceConfig,
        )
        min_instances: int = proto.Field(
            proto.INT32,
            number=5,
            optional=True,
        )
        max_instances: int = proto.Field(
            proto.INT32,
            number=6,
            optional=True,
        )
        resource_limits: MutableMapping[str, str] = proto.MapField(
            proto.STRING,
            proto.STRING,
            number=7,
        )
        container_concurrency: int = proto.Field(
            proto.INT32,
            number=8,
            optional=True,
        )

    class SourceCodeSpec(proto.Message):
        r"""Specification for deploying from source code.

        This message has `oneof`_ fields (mutually exclusive fields).
        For each oneof, at most one member field can be set at the same time.
        Setting any member of the oneof automatically clears all other
        members.

        .. _oneof: https://proto-plus-python.readthedocs.io/en/stable/fields.html#oneofs-mutually-exclusive-fields

        Attributes:
            inline_source (google.cloud.aiplatform_v1.types.ReasoningEngineSpec.SourceCodeSpec.InlineSource):
                Source code is provided directly in the
                request.

                This field is a member of `oneof`_ ``source``.
            developer_connect_source (google.cloud.aiplatform_v1.types.ReasoningEngineSpec.SourceCodeSpec.DeveloperConnectSource):
                Source code is in a Git repository managed by
                Developer Connect.

                This field is a member of `oneof`_ ``source``.
            python_spec (google.cloud.aiplatform_v1.types.ReasoningEngineSpec.SourceCodeSpec.PythonSpec):
                Configuration for a Python application.

                This field is a member of `oneof`_ ``language_spec``.
        """

        class InlineSource(proto.Message):
            r"""Specifies source code provided as a byte stream.

            Attributes:
                source_archive (bytes):
                    Required. Input only. The application source
                    code archive, provided as a compressed tarball
                    (.tar.gz) file.
            """

            source_archive: bytes = proto.Field(
                proto.BYTES,
                number=1,
            )

        class DeveloperConnectConfig(proto.Message):
            r"""Specifies the configuration for fetching source code from a
            Git repository that is managed by Developer Connect. This
            includes the repository, revision, and directory to use.

            Attributes:
                git_repository_link (str):
                    Required. The Developer Connect Git repository link,
                    formatted as
                    ``projects/*/locations/*/connections/*/gitRepositoryLink/*``.
                dir_ (str):
                    Required. Directory, relative to the source
                    root, in which to run the build.
                revision (str):
                    Required. The revision to fetch from the Git
                    repository such as a branch, a tag, a commit
                    SHA, or any Git ref.
            """

            git_repository_link: str = proto.Field(
                proto.STRING,
                number=1,
            )
            dir_: str = proto.Field(
                proto.STRING,
                number=2,
            )
            revision: str = proto.Field(
                proto.STRING,
                number=3,
            )

        class DeveloperConnectSource(proto.Message):
            r"""Specifies source code to be fetched from a Git repository
            managed through the Developer Connect service.

            Attributes:
                config (google.cloud.aiplatform_v1.types.ReasoningEngineSpec.SourceCodeSpec.DeveloperConnectConfig):
                    Required. The Developer Connect configuration
                    that defines the specific repository, revision,
                    and directory to use as the source code root.
            """

            config: "ReasoningEngineSpec.SourceCodeSpec.DeveloperConnectConfig" = (
                proto.Field(
                    proto.MESSAGE,
                    number=1,
                    message="ReasoningEngineSpec.SourceCodeSpec.DeveloperConnectConfig",
                )
            )

        class PythonSpec(proto.Message):
            r"""Specification for running a Python application from source.

            Attributes:
                version (str):
                    Optional. The version of Python to use.
                    Support version includes 3.9, 3.10, 3.11, 3.12,
                    3.13. If not specified, default value is 3.10.
                entrypoint_module (str):
                    Optional. The Python module to load as the
                    entrypoint, specified as a fully qualified
                    module name. For example: path.to.agent. If not
                    specified, defaults to "agent".

                    The project root will be added to Python
                    sys.path, allowing imports to be specified
                    relative to the root.
                entrypoint_object (str):
                    Optional. The name of the callable object within the
                    ``entrypoint_module`` to use as the application If not
                    specified, defaults to "root_agent".
                requirements_file (str):
                    Optional. The path to the requirements file,
                    relative to the source root. If not specified,
                    defaults to "requirements.txt".
            """

            version: str = proto.Field(
                proto.STRING,
                number=1,
            )
            entrypoint_module: str = proto.Field(
                proto.STRING,
                number=2,
            )
            entrypoint_object: str = proto.Field(
                proto.STRING,
                number=3,
            )
            requirements_file: str = proto.Field(
                proto.STRING,
                number=4,
            )

        inline_source: "ReasoningEngineSpec.SourceCodeSpec.InlineSource" = proto.Field(
            proto.MESSAGE,
            number=1,
            oneof="source",
            message="ReasoningEngineSpec.SourceCodeSpec.InlineSource",
        )
        developer_connect_source: (
            "ReasoningEngineSpec.SourceCodeSpec.DeveloperConnectSource"
        ) = proto.Field(
            proto.MESSAGE,
            number=3,
            oneof="source",
            message="ReasoningEngineSpec.SourceCodeSpec.DeveloperConnectSource",
        )
        python_spec: "ReasoningEngineSpec.SourceCodeSpec.PythonSpec" = proto.Field(
            proto.MESSAGE,
            number=2,
            oneof="language_spec",
            message="ReasoningEngineSpec.SourceCodeSpec.PythonSpec",
        )

    source_code_spec: SourceCodeSpec = proto.Field(
        proto.MESSAGE,
        number=11,
        oneof="deployment_source",
        message=SourceCodeSpec,
    )
    service_account: str = proto.Field(
        proto.STRING,
        number=1,
        optional=True,
    )
    package_spec: PackageSpec = proto.Field(
        proto.MESSAGE,
        number=2,
        message=PackageSpec,
    )
    deployment_spec: DeploymentSpec = proto.Field(
        proto.MESSAGE,
        number=4,
        message=DeploymentSpec,
    )
    class_methods: MutableSequence[struct_pb2.Struct] = proto.RepeatedField(
        proto.MESSAGE,
        number=3,
        message=struct_pb2.Struct,
    )
    agent_framework: str = proto.Field(
        proto.STRING,
        number=5,
    )


class ReasoningEngine(proto.Message):
    r"""ReasoningEngine provides a customizable runtime for models to
    determine which actions to take and in which order.

    Attributes:
        name (str):
            Identifier. The resource name of the ReasoningEngine.
            Format:
            ``projects/{project}/locations/{location}/reasoningEngines/{reasoning_engine}``
        display_name (str):
            Required. The display name of the
            ReasoningEngine.
        description (str):
            Optional. The description of the
            ReasoningEngine.
        spec (google.cloud.aiplatform_v1.types.ReasoningEngineSpec):
            Optional. Configurations of the
            ReasoningEngine
        create_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this
            ReasoningEngine was created.
        update_time (google.protobuf.timestamp_pb2.Timestamp):
            Output only. Timestamp when this
            ReasoningEngine was most recently updated.
        etag (str):
            Optional. Used to perform consistent
            read-modify-write updates. If not set, a blind
            "overwrite" update happens.
        encryption_spec (google.cloud.aiplatform_v1.types.EncryptionSpec):
            Customer-managed encryption key spec for a
            ReasoningEngine. If set, this ReasoningEngine
            and all sub-resources of this ReasoningEngine
            will be secured by this key.
        labels (MutableMapping[str, str]):
            Labels for the ReasoningEngine.
    """

    name: str = proto.Field(
        proto.STRING,
        number=1,
    )
    display_name: str = proto.Field(
        proto.STRING,
        number=2,
    )
    description: str = proto.Field(
        proto.STRING,
        number=7,
    )
    spec: "ReasoningEngineSpec" = proto.Field(
        proto.MESSAGE,
        number=3,
        message="ReasoningEngineSpec",
    )
    create_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=4,
        message=timestamp_pb2.Timestamp,
    )
    update_time: timestamp_pb2.Timestamp = proto.Field(
        proto.MESSAGE,
        number=5,
        message=timestamp_pb2.Timestamp,
    )
    etag: str = proto.Field(
        proto.STRING,
        number=6,
    )
    encryption_spec: gca_encryption_spec.EncryptionSpec = proto.Field(
        proto.MESSAGE,
        number=11,
        message=gca_encryption_spec.EncryptionSpec,
    )
    labels: MutableMapping[str, str] = proto.MapField(
        proto.STRING,
        proto.STRING,
        number=17,
    )


__all__ = tuple(sorted(__protobuf__.manifest))



# Copyright 2024 Google LLC All rights reserved.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
from concurrent.futures import ThreadPoolExecutor
from dataclasses import dataclass
from queue import Queue
from typing import Any, TYPE_CHECKING
from threading import Lock, Event

from google.cloud.spanner_v1._opentelemetry_tracing import trace_call
from google.cloud.spanner_v1.metrics.metrics_capture import MetricsCapture

if TYPE_CHECKING:
    from google.cloud.spanner_v1.database import BatchSnapshot

QUEUE_SIZE_PER_WORKER = 32
MAX_PARALLELISM = 16


class PartitionExecutor:
    """
    Executor that executes single partition on a separate thread and inserts
    rows in the queue
    """

    def __init__(
        self, batch_snapshot, partition_id, merged_result_set, lazy_decode=False
    ):
        self._batch_snapshot: BatchSnapshot = batch_snapshot
        self._partition_id = partition_id
        self._merged_result_set: MergedResultSet = merged_result_set
        self._lazy_decode = lazy_decode
        self._queue: Queue[PartitionExecutorResult] = merged_result_set._queue

    def run(self):
        observability_options = getattr(
            self._batch_snapshot, "observability_options", {}
        )
        with trace_call(
            "CloudSpanner.PartitionExecutor.run",
            observability_options=observability_options,
        ), MetricsCapture():
            self.__run()

    def __run(self):
        results = None
        try:
            results = self._batch_snapshot.process_query_batch(
                self._partition_id, lazy_decode=self._lazy_decode
            )
            for row in results:
                if self._merged_result_set._metadata is None:
                    self._set_metadata(results)
                self._queue.put(PartitionExecutorResult(data=row))
            # Special case: The result set did not return any rows.
            # Push the metadata to the merged result set.
            if self._merged_result_set._metadata is None:
                self._set_metadata(results)
        except Exception as ex:
            if self._merged_result_set._metadata is None:
                self._set_metadata(results, True)
            self._queue.put(PartitionExecutorResult(exception=ex))
        finally:
            # Emit a special 'is_last' result to ensure that the MergedResultSet
            # is not blocked on a queue that never receives any more results.
            self._queue.put(PartitionExecutorResult(is_last=True))

    def _set_metadata(self, results, is_exception=False):
        self._merged_result_set.metadata_lock.acquire()
        try:
            if not is_exception:
                self._merged_result_set._metadata = results.metadata
                self._merged_result_set._result_set = results
        finally:
            self._merged_result_set.metadata_lock.release()
            self._merged_result_set.metadata_event.set()


@dataclass
class PartitionExecutorResult:
    data: Any = None
    exception: Exception = None
    is_last: bool = False


class MergedResultSet:
    """
    Executes multiple partitions on different threads and then combines the
    results from multiple queries using a synchronized queue. The order of the
    records in the MergedResultSet is not guaranteed.
    """

    def __init__(
        self, batch_snapshot, partition_ids, max_parallelism, lazy_decode=False
    ):
        self._result_set = None
        self._exception = None
        self._metadata = None
        self.metadata_event = Event()
        self.metadata_lock = Lock()

        partition_ids_count = len(partition_ids)
        self._finished_count_down_latch = partition_ids_count
        parallelism = min(MAX_PARALLELISM, partition_ids_count)
        if max_parallelism != 0:
            parallelism = min(partition_ids_count, max_parallelism)
        self._queue = Queue(maxsize=QUEUE_SIZE_PER_WORKER * parallelism)

        partition_executors = []
        for partition_id in partition_ids:
            partition_executors.append(
                PartitionExecutor(batch_snapshot, partition_id, self, lazy_decode)
            )
        executor = ThreadPoolExecutor(max_workers=parallelism)
        for partition_executor in partition_executors:
            executor.submit(partition_executor.run)
        executor.shutdown(False)

    def __iter__(self):
        return self

    def __next__(self):
        if self._exception is not None:
            raise self._exception
        while True:
            partition_result = self._queue.get()
            if partition_result.is_last:
                self._finished_count_down_latch -= 1
                if self._finished_count_down_latch == 0:
                    raise StopIteration
            elif partition_result.exception is not None:
                self._exception = partition_result.exception
                raise self._exception
            else:
                return partition_result.data

    @property
    def metadata(self):
        self.metadata_event.wait()
        return self._metadata

    @property
    def stats(self):
        # TODO: Implement
        return None

    def decode_row(self, row: []) -> []:
        """Decodes a row from protobuf values to Python objects. This function
           should only be called for result sets that use ``lazy_decoding=True``.
           The array that is returned by this function is the same as the array
           that would have been returned by the rows iterator if ``lazy_decoding=False``.

        :returns: an array containing the decoded values of all the columns in the given row
        """
        if self._result_set is None:
            raise ValueError("iterator not started")
        return self._result_set.decode_row(row)

    def decode_column(self, row: [], column_index: int):
        """Decodes a column from a protobuf value to a Python object. This function
           should only be called for result sets that use ``lazy_decoding=True``.
           The object that is returned by this function is the same as the object
           that would have been returned by the rows iterator if ``lazy_decoding=False``.

        :returns: the decoded column value
        """
        if self._result_set is None:
            raise ValueError("iterator not started")
        return self._result_set.decode_column(row, column_index)



# Copyright 2023 Google LLC
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.
#
"""Classes for working with vision models."""

from vertexai.vision_models._vision_models import (
    GeneratedImage,
    Image,
    ImageCaptioningModel,
    ImageGenerationModel,
    ImageGenerationResponse,
    ImageQnAModel,
    ImageTextModel,
    MultiModalEmbeddingModel,
    MultiModalEmbeddingResponse,
    Video,
    VideoEmbedding,
    VideoSegmentConfig,
)

__all__ = [
    "GeneratedImage",
    "Image",
    "ImageCaptioningModel",
    "ImageGenerationModel",
    "ImageGenerationResponse",
    "ImageQnAModel",
    "ImageTextModel",
    "MultiModalEmbeddingModel",
    "MultiModalEmbeddingResponse",
    "Video",
    "VideoEmbedding",
    "VideoSegmentConfig",
]



"""
Comprehensive tests for BMasterAI including stateful agents and multi-agent coordination
"""

import pytest
import sys
import os
import time
import threading
from unittest.mock import Mock, patch

# Add src to path for testing
sys.path.insert(0, os.path.join(os.path.dirname(__file__), '..', 'src'))

from bmasterai.logging import configure_logging, LogLevel, EventType
from bmasterai.monitoring import get_monitor
from bmasterai.integrations import get_integration_manager


class TestStatefulAgent:
    """Test the StatefulAgent functionality"""

    def setup_method(self):
        """Setup for each test"""
        # Import here to avoid circular imports
        from examples.enhanced_examples import StatefulAgent
        self.StatefulAgent = StatefulAgent

        # Configure logging for tests
        configure_logging(log_level=LogLevel.INFO)

    def test_agent_creation(self):
        """Test agent creation and initialization"""
        agent = self.StatefulAgent("test-agent", "TestAgent", {"test": True})

        assert agent.agent_id == "test-agent"
        assert agent.name == "TestAgent"
        assert agent.state.agent_id == "test-agent"
        assert agent.state.status == "initialized"
        assert agent.state.metadata["test"] is True
        assert len(agent.task_history) == 0
        assert len(agent.memory) == 0

    def test_agent_start_stop(self):
        """Test agent start and stop functionality"""
        agent = self.StatefulAgent("test-agent", "TestAgent")

        # Test start
        agent.start()
        assert agent.running is True
        assert agent.state.status == "running"
        assert agent.thread is not None

        # Test stop
        agent.stop()
        assert agent.running is False
        assert agent.state.status == "stopped"

    def test_data_processing_task(self):
        """Test data processing task execution"""
        agent = self.StatefulAgent("test-agent", "TestAgent")
        agent.start()

        try:
            # Test sum operation
            result = agent.execute_task("data_processing", {
                "data": [1, 2, 3, 4, 5],
                "operation": "sum"
            })

            assert result["success"] is True
            assert result["result"]["result"] == 15
            assert result["result"]["operation"] == "sum"
            assert agent.state.task_count == 1

            # Test average operation
            result = agent.execute_task("data_processing", {
                "data": [10, 20, 30],
                "operation": "average"
            })

            assert result["success"] is True
            assert result["result"]["result"] == 20.0
            assert agent.state.task_count == 2

        finally:
            agent.stop()

    def test_memory_operations(self):
        """Test agent memory operations"""
        agent = self.StatefulAgent("test-agent", "TestAgent")
        agent.start()

        try:
            # Test set operation
            result = agent.execute_task("memory_operation", {
                "operation": "set",
                "key": "test_key",
                "value": "test_value"
            })

            assert result["success"] is True
            assert result["result"]["set"] is True
            assert agent.memory["test_key"] == "test_value"

            # Test get operation
            result = agent.execute_task("memory_operation", {
                "operation": "get",
                "key": "test_key"
            })

            assert result["success"] is True
            assert result["result"]["value"] == "test_value"

            # Test size operation
            result = agent.execute_task("memory_operation", {
                "operation": "size"
            })

            assert result["success"] is True
            assert result["result"]["memory_size"] > 0  # Should have processing results + test_key

        finally:
            agent.stop()

    def test_state_persistence(self):
        """Test that agent state persists across tasks"""
        agent = self.StatefulAgent("test-agent", "TestAgent")
        agent.start()

        try:
            # Execute multiple tasks
            for i in range(3):
                agent.execute_task("data_processing", {
                    "data": [i, i+1, i+2],
                    "operation": "sum"
                })

            # Check state
            state = agent.get_state_summary()
            assert state["task_count"] == 3
            assert len(state["recent_tasks"]) == 3
            assert len(agent.memory) > 0  # Should have processing results

        finally:
            agent.stop()


class TestMultiAgentOrchestrator:
    """Test the MultiAgentOrchestrator functionality"""

    def setup_method(self):
        """Setup for each test"""
        from examples.enhanced_examples import StatefulAgent, MultiAgentOrchestrator
        self.StatefulAgent = StatefulAgent
        self.MultiAgentOrchestrator = MultiAgentOrchestrator

        configure_logging(log_level=LogLevel.INFO)

    def test_orchestrator_creation(self):
        """Test orchestrator creation"""
        orchestrator = self.MultiAgentOrchestrator("test-orchestrator")

        assert orchestrator.orchestrator_id == "test-orchestrator"
        assert len(orchestrator.agents) == 0
        assert len(orchestrator.coordination_history) == 0

    def test_agent_management(self):
        """Test adding and removing agents"""
        orchestrator = self.MultiAgentOrchestrator("test-orchestrator")
        agent1 = self.StatefulAgent("agent-1", "Agent1")
        agent2 = self.StatefulAgent("agent-2", "Agent2")

        # Test adding agents
        orchestrator.add_agent(agent1)
        orchestrator.add_agent(agent2)

        assert len(orchestrator.agents) == 2
        assert "agent-1" in orchestrator.agents
        assert "agent-2" in orchestrator.agents

        # Test removing agent
        orchestrator.remove_agent("agent-1")
        assert len(orchestrator.agents) == 1
        assert "agent-1" not in orchestrator.agents

    def test_agent_lifecycle_management(self):
        """Test starting and stopping all agents"""
        orchestrator = self.MultiAgentOrchestrator("test-orchestrator")
        agent1 = self.StatefulAgent("agent-1", "Agent1")
        agent2 = self.StatefulAgent("agent-2", "Agent2")

        orchestrator.add_agent(agent1)
        orchestrator.add_agent(agent2)

        # Test starting all agents
        started_count = orchestrator.start_all_agents()
        assert started_count == 2
        assert agent1.running is True
        assert agent2.running is True

        # Test stopping all agents
        stopped_count = orchestrator.stop_all_agents()
        assert stopped_count == 2
        assert agent1.running is False
        assert agent2.running is False

    def test_task_coordination(self):
        """Test coordinating tasks across multiple agents"""
        orchestrator = self.MultiAgentOrchestrator("test-orchestrator")
        agent1 = self.StatefulAgent("agent-1", "Agent1")
        agent2 = self.StatefulAgent("agent-2", "Agent2")

        orchestrator.add_agent(agent1)
        orchestrator.add_agent(agent2)
        orchestrator.start_all_agents()

        try:
            # Define task assignments
            task_assignments = {
                "agent-1": ("data_processing", {
                    "data": [1, 2, 3],
                    "operation": "sum"
                }),
                "agent-2": ("memory_operation", {
                    "operation": "set",
                    "key": "test",
                    "value": "coordination_test"
                })
            }

            # Coordinate tasks
            result = orchestrator.coordinate_task("test-coordination", task_assignments)

            assert result["success"] is True
            assert result["coordination_id"] == "test-coordination"
            assert len(result["results"]) == 2
            assert "agent-1" in result["results"]
            assert "agent-2" in result["results"]
            assert len(result["errors"]) == 0

            # Check coordination history
            assert len(orchestrator.coordination_history) == 1

        finally:
            orchestrator.stop_all_agents()

    def test_orchestrator_status(self):
        """Test getting orchestrator status"""
        orchestrator = self.MultiAgentOrchestrator("test-orchestrator")
        agent = self.StatefulAgent("agent-1", "Agent1")

        orchestrator.add_agent(agent)
        orchestrator.start_all_agents()

        try:
            # Execute a task to generate some activity
            agent.execute_task("data_processing", {
                "data": [1, 2, 3],
                "operation": "sum"
            })

            # Get status
            status = orchestrator.get_orchestrator_status()

            assert status["orchestrator_id"] == "test-orchestrator"
            assert status["total_agents"] == 1
            assert status["running_agents"] == 1
            assert "agent-1" in status["agent_statuses"]
            assert status["agent_statuses"]["agent-1"]["task_count"] == 1

        finally:
            orchestrator.stop_all_agents()


class TestLoggingIntegration:
    """Test logging integration with agents"""

    def test_logging_configuration(self):
        """Test basic logging configuration"""
        logger = configure_logging(log_level=LogLevel.INFO)
        assert logger is not None

    def test_agent_logging_events(self):
        """Test that agents generate proper logging events"""
        from examples.enhanced_examples import StatefulAgent

        configure_logging(log_level=LogLevel.INFO)
        agent = StatefulAgent("test-agent", "TestAgent")

        # Mock the logger to capture events
        with patch.object(agent.logger, 'log_event') as mock_log:
            agent.start()
            agent.execute_task("data_processing", {"data": [1, 2, 3], "operation": "sum"})
            agent.stop()

            # Verify logging events were called
            assert mock_log.call_count >= 3  # At least start, task, stop events

            # Check for specific event types
            event_types = [call[0][1] for call in mock_log.call_args_list]  # event_type is the second parameter
            assert EventType.AGENT_START in event_types
            assert EventType.TASK_START in event_types
            assert EventType.TASK_COMPLETE in event_types
            assert EventType.AGENT_STOP in event_types


class TestMonitoringIntegration:
    """Test monitoring integration with agents"""

    def test_monitor_initialization(self):
        """Test monitor initialization"""
        monitor = get_monitor()
        assert monitor is not None

    def test_agent_registration_with_monitor(self):
        """Test that agents register with the monitor"""
        from examples.enhanced_examples import StatefulAgent

        configure_logging(log_level=LogLevel.INFO)
        monitor = get_monitor()

        agent = StatefulAgent("test-agent", "TestAgent")

        # Mock the monitor to capture agent tracking
        with patch.object(monitor, 'track_agent_start') as mock_track_start:
            agent.start()
            mock_track_start.assert_called_once_with("test-agent")
            agent.stop()


def test_integration_manager():
    """Test integration manager initialization"""
    manager = get_integration_manager()
    assert manager is not None


if __name__ == "__main__":
    pytest.main([__file__, "-v"])
